@@ -1,8 +1,9 @@
-# Copyright 2022 MosaicML Streaming authors
+# Copyright 2023 MosaicML Streaming authors
 # SPDX-License-Identifier: Apache-2.0
 
 """MosaicML Streaming Datasets for cloud-native model training."""
 
+import streaming.multimodal as multimodal
 import streaming.text as text
 import streaming.vision as vision
 from streaming._version import __version__
@@ -11,5 +12,5 @@ from streaming.base import (CSVWriter, JSONWriter, LocalDataset, MDSWriter, Stre
 
 __all__ = [
     'StreamingDataLoader', 'StreamingDataset', 'CSVWriter', 'JSONWriter', 'MDSWriter', 'TSVWriter',
-    'XSVWriter', 'LocalDataset', 'vision', 'text'
+    'XSVWriter', 'LocalDataset', 'multimodal', 'vision', 'text'
 ]
