@@ -26,7 +26,7 @@ from streaming.base.index import Index, get_index_basename
 from streaming.base.partitioning import get_partitions
 from streaming.base.shared import SharedBarrier, create_shared_memory
 from streaming.base.shuffle import get_shuffle
-from streaming.base.storage import download
+from streaming.base.storage import download_file
 from streaming.base.util import wait_for_file_to_exist, wait_for_local_leader
 from streaming.base.world import World
 
@@ -122,13 +122,17 @@ class StreamingDataset(IterableDataset):
         validate_hash (str, optional): Optional hash or checksum algorithm to use to validate
             shards. Defaults to ``None``.
         shuffle_seed (int): Seed for Deterministic data shuffling. Defaults to ``9176``.
-        num_canonical_nodes (int, optional): Canonical number of nodes for shuffling with resumption.
-            Defaults to ``None``, which is interpreted as the number of nodes of the initial run.
+        num_canonical_nodes (int, optional): Canonical number of nodes for shuffling with
+            resumption. Defaults to ``None``, which is interpreted as the number of nodes of the
+            initial run.
         batch_size (int, optional): Batch size of its DataLoader, which affects how the dataset is
             partitioned over the workers. Defaults to ``None``.
+        partition_algo (str): Which partitioning algorithm to use. Defaults to ``orig``.
+        shuffle_algo (str): Which shuffling algorithm to use. Defaults to ``py2s``.
     """
 
     def __init__(self,
+                 *,
                  local: str,
                  remote: Optional[str] = None,
                  split: Optional[str] = None,
@@ -140,7 +144,9 @@ class StreamingDataset(IterableDataset):
                  validate_hash: Optional[str] = None,
                  shuffle_seed: int = 9176,
                  num_canonical_nodes: Optional[int] = None,
-                 batch_size: Optional[int] = None):
+                 batch_size: Optional[int] = None,
+                 partition_algo: str = 'orig',
+                 shuffle_algo: str = 'py2s') -> None:
         self.local = local
         self.remote = remote
         self.split = split or ''  # Empty string for os.path.join().
@@ -150,6 +156,8 @@ class StreamingDataset(IterableDataset):
         self.download_retry = download_retry
         self.download_timeout = download_timeout
         self.validate_hash = validate_hash or None
+        self.partition_algo = partition_algo
+        self.shuffle_algo = shuffle_algo
 
         if self.download_retry < 0:
             raise ValueError('Parameter ``download_retry`` must be non-negative')
@@ -207,8 +215,8 @@ class StreamingDataset(IterableDataset):
         prefix_int = int(seed_rng.integers(1 << 24))
         self._prefix = f'{prefix_int:06x}'
 
-        # Should be a unique shared directory per each StreamingDataset instantiation to avoid a conflict
-        # between a different StreamingDataset instance on a same machine.
+        # Should be a unique shared directory per each StreamingDataset instantiation to avoid a
+        # conflict between a different StreamingDataset instance on a same machine.
         start_time = time()
         while True:
             self._shared_dir = os.path.join(os.path.sep, 'tmp', 'streaming', self._prefix)
@@ -229,13 +237,14 @@ class StreamingDataset(IterableDataset):
 
         # Initialize the distributed package and synchronize all the ranks
         is_dist_pg_initialized = False
-        if dist.is_available() and not dist.is_initialized():
-            is_dist_pg_initialized = True
-            dist.init_process_group(backend='nccl' if torch.cuda.is_available() and
-                                    dist.is_nccl_available() else 'gloo',
-                                    rank=world.rank,
-                                    world_size=world.num_ranks)
-        dist.barrier()
+        if self._rank_world.num_ranks > 1:
+            if dist.is_available() and not dist.is_initialized():
+                is_dist_pg_initialized = True
+                dist.init_process_group(backend='nccl' if torch.cuda.is_available() and
+                                        dist.is_nccl_available() else 'gloo',
+                                        rank=world.rank,
+                                        world_size=world.num_ranks)
+            dist.barrier()
 
         # Create the shared memory-backed worker barrier, without its lock, which is unpickleable.
         worker_barrier_filelock_path = os.path.join(self._shared_dir, 'barrier_filelock')
@@ -401,8 +410,8 @@ class StreamingDataset(IterableDataset):
                                         world.num_nodes, world.ranks_per_node,
                                         world.workers_per_rank, self.batch_size, sample_in_epoch)
             if self.shuffle:
-                mapping = get_shuffle(self.shard_sizes, self.num_canonical_nodes,
-                                      self.shuffle_seed, epoch)
+                mapping = get_shuffle(self.shuffle_algo, self.shard_sizes,
+                                      self.num_canonical_nodes, self.shuffle_seed, epoch)
                 sample_ids = np.where(sample_ids == -1, -1, mapping[sample_ids])
             sample_ids.tofile(tmp_filename)
             os.rename(tmp_filename, filename)
@@ -457,7 +466,7 @@ class StreamingDataset(IterableDataset):
         errors = []
         for _ in range(1 + self.download_retry):
             try:
-                download(remote, local, self.download_timeout)
+                download_file(remote, local, self.download_timeout)
             except FileNotFoundError:  # Bubble up file not found error.
                 raise
             except Exception as e:  # Retry for all other causes of failure.
