@@ -4,25 +4,39 @@
 """Serialize samples into streaming dataset shards and index."""
 
 import json
+import logging
 import os
+import shutil
 from abc import ABC, abstractmethod
+from concurrent.futures import ThreadPoolExecutor
 from types import TracebackType
-from typing import Any, Dict, List, Optional, Tuple, Type
+from typing import Any, Dict, List, Optional, Tuple, Type, Union
 
 from typing_extensions import Self
 
 from streaming.base.compression import compress, get_compression_extension, is_compression
 from streaming.base.hashing import get_hash, is_hash
 from streaming.base.index import get_index_basename
+from streaming.base.storage.upload import CloudUploader
 
 __all__ = ['JointWriter', 'SplitWriter']
 
+logger = logging.getLogger(__name__)
+
 
 class Writer(ABC):
     """Writes a streaming dataset.
 
     Args:
-        dirname (str): Local dataset directory.
+        out (str | Tuple[str, str]): Output dataset directory to save shard files.
+            1. If `out` is a local directory, shard files are saved locally.
+            2. If `out` is a remote directory, a local temporary directory is created to
+                cache the shard files and then the shard files are uploaded to a remote
+                location. At the end, the temp directory is deleted once shards are uploaded.
+            3. If `out` is a tuple of `(local_dir, remote_dir)`, shard files are saved in the
+                `local_dir` and also uploaded to a remote location.
+        keep_local (bool): If the dataset is uploaded, whether to keep the local dataset directory
+            or remove it after uploading. Defaults to ``False``.
         compression (str, optional): Optional compression or compression:level. Defaults to
             ``None``.
         hashes (List[str], optional): Optional list of hash algorithms to apply to shard files.
@@ -33,32 +47,41 @@ class Writer(ABC):
             while writing). Defaults to ``0``.
         extra_bytes_per_sample (int): Extra bytes per serialized sample (for computing shard size
             while writing). Defaults to ``0``.
+        **kwargs (Any): Additional settings for the Writer.
     """
 
     format: str = ''  # Name of the format (like "mds", "csv", "json", etc).
 
     def __init__(self,
-                 dirname: str,
+                 *,
+                 out: Union[str, Tuple[str, str]],
+                 keep_local: bool = False,
                  compression: Optional[str] = None,
                  hashes: Optional[List[str]] = None,
                  size_limit: Optional[int] = 1 << 26,
                  extra_bytes_per_shard: int = 0,
-                 extra_bytes_per_sample: int = 0) -> None:
+                 extra_bytes_per_sample: int = 0,
+                 **kwargs: Any) -> None:
+
         compression = compression or None
         if compression:
-            assert is_compression(compression)
+            if not is_compression(compression):
+                raise ValueError('Invalid compression: {compression}.')
 
         hashes = hashes or []
-        assert list(hashes) == sorted(hashes)
+        if list(hashes) != sorted(hashes):
+            raise ValueError('Hashes must be unique and in sorted order.')
         for algo in hashes:
-            assert is_hash(algo)
+            if not is_hash(algo):
+                raise ValueError('Invalid hash: {algo}.')
 
         if size_limit:
-            assert 0 < size_limit
+            if size_limit < 0:
+                raise ValueError('Size limit, if provided, must be greater than zero.')
         else:
             size_limit = None
 
-        self.dirname = dirname
+        self.keep_local = keep_local
         self.compression = compression
         self.hashes = hashes
         self.size_limit = size_limit
@@ -69,10 +92,12 @@ class Writer(ABC):
 
         self.shards = []
 
-        # Raise an exception if the directory is not empty
-        if os.path.exists(dirname) and len(os.listdir(dirname)) != 0:
-            raise FileExistsError(f'Directory is not empty: {dirname}')
-        os.makedirs(dirname, exist_ok=True)
+        self.cloud_writer = CloudUploader.get(out, keep_local, kwargs.get('progress_bar', False))
+        self.local = self.cloud_writer.local
+        self.remote = self.cloud_writer.remote
+        # `max_workers`: The maximum number of threads that can be executed in parallel.
+        # One thread is responsible for uploading one shard file to a remote location.
+        self.executor = ThreadPoolExecutor(max_workers=kwargs.get('max_workers', None))
 
         self._reset_cache()
 
@@ -155,7 +180,7 @@ class Writer(ABC):
             zip_info = None
             data = raw_data
             basename = raw_basename
-        filename = os.path.join(self.dirname, basename)
+        filename = os.path.join(self.local, basename)
         with open(filename, 'wb') as out:
             out.write(data)
         return raw_info, zip_info
@@ -197,14 +222,17 @@ class Writer(ABC):
 
     def _write_index(self) -> None:
         """Write the index, having written all the shards."""
-        assert not self.new_samples
-        filename = os.path.join(self.dirname, get_index_basename())
+        if self.new_samples:
+            raise RuntimeError('Internal error: not all samples have been written.')
+        basename = get_index_basename()
+        filename = os.path.join(self.local, basename)
         obj = {
             'version': 2,
             'shards': self.shards,
         }
         with open(filename, 'w') as out:
             json.dump(obj, out, sort_keys=True)
+        self.executor.submit(self.cloud_writer.upload_file, basename)
 
     def finish(self) -> None:
         """Finish writing samples."""
@@ -212,6 +240,10 @@ class Writer(ABC):
             self.flush_shard()
             self._reset_cache()
         self._write_index()
+        logger.debug(f'Waiting for all shard files to get uploaded to {self.remote}')
+        self.executor.shutdown(wait=True, cancel_futures=False)
+        if self.remote and not self.keep_local:
+            shutil.rmtree(self.local)
 
     def __enter__(self) -> Self:
         """Enter context manager.
@@ -237,7 +269,15 @@ class JointWriter(Writer):
     """Writes a streaming dataset with joint shards.
 
     Args:
-        dirname (str): Local dataset directory.
+        out (str | Tuple[str, str]): Output dataset directory to save shard files.
+            1. If `out` is a local directory, shard files are saved locally.
+            2. If `out` is a remote directory, a local temporary directory is created to
+                cache the shard files and then the shard files are uploaded to a remote
+                location. At the end, the temp directory is deleted once shards are uploaded.
+            3. If `out` is a tuple of `(local_dir, remote_dir)`, shard files are saved in the
+                `local_dir` and also uploaded to a remote location.
+        keep_local (bool): If the dataset is uploaded, whether to keep the local dataset directory
+            or remove it after uploading. Defaults to ``False``.
         compression (str, optional): Optional compression or compression:level. Defaults to
             ``None``.
         hashes (List[str], optional): Optional list of hash algorithms to apply to shard files.
@@ -248,17 +288,27 @@ class JointWriter(Writer):
             while writing). Defaults to ``0``.
         extra_bytes_per_sample (int): Extra bytes per serialized sample (for computing shard size
             while writing). Defaults to ``0``.
+        **kwargs (Any): Additional settings for the Writer.
     """
 
     def __init__(self,
-                 dirname: str,
+                 *,
+                 out: Union[str, Tuple[str, str]],
+                 keep_local: bool = False,
                  compression: Optional[str] = None,
                  hashes: Optional[List[str]] = None,
                  size_limit: Optional[int] = 1 << 26,
                  extra_bytes_per_shard: int = 0,
-                 extra_bytes_per_sample: int = 0) -> None:
-        super().__init__(dirname, compression, hashes, size_limit, extra_bytes_per_shard,
-                         extra_bytes_per_sample)
+                 extra_bytes_per_sample: int = 0,
+                 **kwargs: Any) -> None:
+        super().__init__(out=out,
+                         keep_local=keep_local,
+                         compression=compression,
+                         hashes=hashes,
+                         size_limit=size_limit,
+                         extra_bytes_per_shard=extra_bytes_per_shard,
+                         extra_bytes_per_sample=extra_bytes_per_sample,
+                         **kwargs)
 
     @abstractmethod
     def encode_joint_shard(self) -> bytes:
@@ -282,6 +332,8 @@ class JointWriter(Writer):
         obj.update(self.get_config())
         self.shards.append(obj)
 
+        self.executor.submit(self.cloud_writer.upload_file, zip_data_basename or raw_data_basename)
+
 
 class SplitWriter(Writer):
     """Writes a streaming dataset with split shards.
@@ -289,25 +341,43 @@ class SplitWriter(Writer):
     Split shards refer to raw data (csv, json, etc.) paired with an index into it.
 
     Args:
-        dirname (str): Local dataset directory.
+        out (str | Tuple[str, str]): Output dataset directory to save shard files.
+            1. If `out` is a local directory, shard files are saved locally.
+            2. If `out` is a remote directory, a local temporary directory is created to
+                cache the shard files and then the shard files are uploaded to a remote
+                location. At the end, the temp directory is deleted once shards are uploaded.
+            3. If `out` is a tuple of `(local_dir, remote_dir)`, shard files are saved in the
+                `local_dir` and also uploaded to a remote location.
+        keep_local (bool): If the dataset is uploaded, whether to keep the local dataset directory
+            or remove it after uploading. Defaults to ``False``.
         compression (str, optional): Optional compression or compression:level. Defaults to
             ``None``.
         hashes (List[str], optional): Optional list of hash algorithms to apply to shard files.
             Defaults to ``None``.
         size_limit (int, optional): Optional shard size limit, after which point to start a new
             shard. If None, puts everything in one shard. Defaults to ``1 << 26``.
+        **kwargs (Any): Additional settings for the Writer.
     """
 
     extra_bytes_per_shard = 0
     extra_bytes_per_sample = 0
 
     def __init__(self,
-                 dirname: str,
+                 *,
+                 out: Union[str, Tuple[str, str]],
+                 keep_local: bool = False,
                  compression: Optional[str] = None,
                  hashes: Optional[List[str]] = None,
-                 size_limit: Optional[int] = 1 << 26) -> None:
-        super().__init__(dirname, compression, hashes, size_limit, self.extra_bytes_per_shard,
-                         self.extra_bytes_per_sample)
+                 size_limit: Optional[int] = 1 << 26,
+                 **kwargs: Any) -> None:
+        super().__init__(out=out,
+                         keep_local=keep_local,
+                         compression=compression,
+                         hashes=hashes,
+                         size_limit=size_limit,
+                         extra_bytes_per_shard=self.extra_bytes_per_shard,
+                         extra_bytes_per_sample=self.extra_bytes_per_sample,
+                         **kwargs)
 
     @abstractmethod
     def encode_split_shard(self) -> Tuple[bytes, bytes]:
@@ -320,12 +390,12 @@ class SplitWriter(Writer):
 
     def flush_shard(self) -> None:
         raw_data_basename, zip_data_basename = self._name_next_shard()
-        meta_raw_basename, meta_zip_basename = self._name_next_shard('meta')
+        raw_meta_basename, zip_meta_basename = self._name_next_shard('meta')
         raw_data, raw_meta = self.encode_split_shard()
         raw_data_info, zip_data_info = self._process_file(raw_data, raw_data_basename,
                                                           zip_data_basename)
-        raw_meta_info, zip_meta_info = self._process_file(raw_meta, meta_raw_basename,
-                                                          meta_zip_basename)
+        raw_meta_info, zip_meta_info = self._process_file(raw_meta, raw_meta_basename,
+                                                          zip_meta_basename)
         obj = {
             'samples': len(self.new_samples),
             'raw_data': raw_data_info,
@@ -335,3 +405,6 @@ class SplitWriter(Writer):
         }
         obj.update(self.get_config())
         self.shards.append(obj)
+
+        self.executor.submit(self.cloud_writer.upload_file, zip_data_basename or raw_data_basename)
+        self.executor.submit(self.cloud_writer.upload_file, zip_meta_basename or raw_meta_basename)
