@@ -1,10 +1,10 @@
 # Copyright 2023 MosaicML Streaming authors
 # SPDX-License-Identifier: Apache-2.0
 
-""":class:`MDSWriter` converts a list of samples into binary `.mds` files that can be read as a :class:`MDSReader`."""
+""":class:`MDSWriter` writes samples to ``.mds`` files that can be read by :class:`MDSReader`."""
 
 import json
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 import numpy as np
 
@@ -19,27 +19,44 @@ class MDSWriter(JointWriter):
     """Writes a streaming MDS dataset.
 
     Args:
-        dirname (str): Local dataset directory.
         columns (Dict[str, str]): Sample columns.
+        out (str | Tuple[str, str]): Output dataset directory to save shard files.
+            1. If `out` is a local directory, shard files are saved locally.
+            2. If `out` is a remote directory, a local temporary directory is created to
+                cache the shard files and then the shard files are uploaded to a remote
+                location. At the end, the temp directory is deleted once shards are uploaded.
+            3. If `out` is a tuple of `(local_dir, remote_dir)`, shard files are saved in the
+                `local_dir` and also uploaded to a remote location.
+        keep_local (bool): If the dataset is uploaded, whether to keep the local dataset directory
+            or remove it after uploading. Defaults to ``False``.
         compression (str, optional): Optional compression or compression:level. Defaults to
             ``None``.
         hashes (List[str], optional): Optional list of hash algorithms to apply to shard files.
             Defaults to ``None``.
         size_limit (int, optional): Optional shard size limit, after which point to start a new
-            shard. If None, puts everything in one shard. Defaults to ``1 << 26``.
+            shard. If ``None``, puts everything in one shard. Defaults to ``1 << 26``.
+        **kwargs (Any): Additional settings for the Writer.
     """
 
     format = 'mds'
     extra_bytes_per_sample = 4
 
     def __init__(self,
-                 dirname: str,
+                 *,
                  columns: Dict[str, str],
+                 out: Union[str, Tuple[str, str]],
+                 keep_local: bool = False,
                  compression: Optional[str] = None,
                  hashes: Optional[List[str]] = None,
-                 size_limit: Optional[int] = 1 << 26) -> None:
-        super().__init__(dirname, compression, hashes, size_limit, 0, self.extra_bytes_per_sample)
-
+                 size_limit: Optional[int] = 1 << 26,
+                 **kwargs: Any) -> None:
+        super().__init__(out=out,
+                         keep_local=keep_local,
+                         compression=compression,
+                         hashes=hashes,
+                         size_limit=size_limit,
+                         extra_bytes_per_sample=self.extra_bytes_per_sample,
+                         **kwargs)
         self.columns = columns
         self.column_names = []
         self.column_encodings = []
@@ -47,9 +64,8 @@ class MDSWriter(JointWriter):
         for name in sorted(columns):
             encoding = columns[name]
             if not is_mds_encoding(encoding):
-                raise TypeError(
-                    f'MDSWriter passed column "{name}" with encoding "{encoding}" is unsupported. Supported encodings are {get_mds_encodings()}'
-                )
+                raise TypeError(f'MDSWriter passed column "{name}" with encoding "{encoding}" ' +
+                                f'is unsupported. Supported encodings are {get_mds_encodings()}')
             size = get_mds_encoded_size(encoding)
             self.column_names.append(name)
             self.column_encodings.append(encoding)
@@ -81,9 +97,8 @@ class MDSWriter(JointWriter):
                 sizes.append(size)
             else:
                 if size != len(datum):
-                    raise KeyError(
-                        f'Unexpected data size; was this data typed with the correct encoding ({encoding})?'
-                    )
+                    raise KeyError(f'Unexpected data size; was this data typed with the correct ' +
+                                   f'encoding ({encoding})?')
             data.append(datum)
         head = np.array(sizes, np.uint32).tobytes()
         body = b''.join(data)
